sr		= 44100
kr		= 4410
nchnls		= 8

gimach		= 344.0			;speed of sound
gamix 		init 0			;global variable for reverb send	
gifrontwall	= 15.0			; position of frontwall at the x-axis [units]
gisidewall	= 15.0			; position of sidewall at the y-axis [units]
giceiling	= 8.0			; height of the ceiling measured from the zero-point of the coordinate-system [units]
giunit		= 1			; radius of the "unit- sphere" (distance of the speaker from the center- point [m])
					
					; Global Reverb Settings, conceived and coded 1999-2005 by Jan Jacob Hofmann
girvbdecay	= 0.9			; gain of reverb. Adjust empirically for desired reverb time. .6 gives	
	               			; a good small "live" room sound, .8 a small hall, .9 a large hall, .99 an enormous stone cavern.	
girandpchmod	= 1.0		 	; amount of random pitch modulation for the delay lines. 1 is the "normal"	
                			; amount, but this may be too high for held pitches such as piano tones. Adjust to taste.		
gicutofffreq    = 15000		     	; Cutoff frequency of lowpass filters in feedback loops of delay lines,		
                			; in Hz. Lower cutoff frequencies results in a sound with more high-frequency damping.		

instr 1
asrw init 0	;storage for diffuse reflections
asrx init 0
asry init 0
asrz init 0
asrr init 0
asrs init 0
asrt init 0
asru init 0
asrv init 0
assrw init 0	;storage for specular reflections
assrx init 0
assry init 0
assrz init 0
assrr init 0
assrs init 0
assrt init 0
assru init 0
assrv init 0

iAold 	=	p6
iA 	=	p7
iEold 	=	p8
iE 	=	p9
iDold 	=	p10
iD 	=	p11

iAold	=	((abs(iA-iAold))>180.0 ? iA+1:iAold) ; ensure no 0 to 360 degree jumps.       

kA line iAold, p3  ,iA				;generate control value changing from old to new spatial coordinate	
kE line iEold, p3  ,iE
kD line iDold, p3  ,iD

ilen 	filelen "A1.wav" 			;get length in secs of source file
igpt = 	p13*(ilen-(p3*p14))			;add random offset (file duration -(grain dur x play speed);
andx1	line	igpt,p12,igpt+p12		;generate table index
andx = 	andx1*p14				;multiply by play speed
asig	tablei	andx*sr,1			;read audio from table
asig =	asig*10000
k1  	oscil1i 0, 1, p12, 2  			; generate grain envelope;
k2   = 	k1-0.43868 				; remove dc offset?
agrain =k2*asig					; generate grain

kD = (kD*4) 					; kD goes from 0 -8, extend range
kDfac = 1/kD					; Distance scaling factor
kDfac2 = (kDfac>= 1.0 ? 1.0:kDfac) 		; ensure no infinity distance factor values.

kDfacR = ((kD/16)*-1)+1				; Distance scaling factor for reverb send
kDfacR2 = (kDfacR<= 0.0 ? 0.0:kDfacR)		; Ensure no negative values
agrain2 = (agrain * kDfac2)			; Scale grain according to distance
agrainr=(agrain*0.005)*kDfacR2			; Calculate reverb send grain
kfreq=(22000*kDfac2)				; Cut off freq for LPF
agrain3 tone agrain2, kfreq, 0  		; Model air absorption
						; Doppler code from Doppler1.orc by Christopher Dobrian
adelayline	delayr	1 			; create a 1-second delay line
adelgrain	deltapi	((kD/gimach)+ 0.01)	; look a certain time into the delay line,
						; based on distance of the sound source
delayw	agrain3 				; this is what is written into the delay line

;**************************************************************************************************************************
; DIFFUSE  EARLY REFLECTIONS 	conceived and coded 1999-2001 by Jan Jacob Hofmann, using the Furse-Malham-Set (FMH) of encoding equations.
; Thanks to Dave Malham and Richard Furse. Thanks to Ingo. 
; Based on a Paper by David Griesinger published in the Proceedings of the 19th  AES International Conference, June 2001.
;**************************************************************************************************************************
idamp 		= 0.1				; damping factor of the reflecting surface (gain of reverb)
ihighdamp 	= 15000				; absorbtion characteristics of the wall in respect to high frequencies: frequencies above ihighdamp get filtered out
ilfboost	= 1				; low frequency boost for sources near a wall

kx 		= kD * cos(kA) * cos(kE)	; Position of sound in kartesian coordinates
ky 		= kD * sin(kA) * cos(kE)	; Position of sound in kartesian coordinates
kz		= kD * sin(kE)			; Position of sound in kartesian coordinates
kx		= (kx <	0.01 && kx > -0.01 ? 0.01  :kx)		; fix for that strange y-axis noise

arefsig		tone	adelgrain, ihighdamp	; filtering of high frequencies due to absorbtion characteristics of the wall
	
#define	DIFFREFLECTION(R)#

icnt	= $R.

krefx		= ( icnt= 1  	? 	gifrontwall	: 	kx)		; asigning a position of the reflection at the x-axis
if 	icnt	= 1	kgoto	yrefcalc$R.
krefx		= ( icnt= 2		? 	-gifrontwall: 	kx)		; asigning a position of the reflection at the x-axis

yrefcalc$R.:
krefy 	= ( icnt= 3 	? 	gisidewall	: 	ky)			; asigning a position of the reflection at the y-axis
if 	icnt	= 3	kgoto	zrefcalc$R.
krefy 	= ( icnt= 4		? 	-gisidewall	: 	ky)		; asigning a position of the reflection at the y-axis 

zrefcalc$R.:
krefz		= ( icnt= 5  	? 	giceiling	: 	kz)		; asigning a position of the reflection at the z-axis
if 	icnt	= 5	kgoto	refout$R.
krefz		= ( icnt= 6		? 	-giceiling	: 	kz)	; asigning a position of the reflection at the z-axis
refout$R.:

; A, E, D - calculation of the reflecting point
kDref		= sqrt(krefx^2+ krefy^2+ krefz^2)				; calculates the distance of the reflection
kxref		= krefx / kDref							; this operation generates a vector equivalent at the unit sphere
kyref		= krefy / kDref							; this operation generates a vector equivalent at the unit sphere
kzref		= krefz / kDref							; this operation generates a vector equivalent at the unit sphere
; Elevation calculation
kEref		=	cosinv ( sqrt( 1- kzref^2))				; calculation of Elevation
kEref		= 	(kzref	< 0  	?  	- kEref  : kEref)		; if z is negative make Elevation- vector negative too
;  Angle calculation
kAref		= sininv (kyref/ cos (kEref))					; calculation of Angle
kAref		= (kxref  >=  0  ?   	kAref : 3.1415 - kAref)			; distinction of positive and negative y
kAref		= (kyref  <=  0   &&	kxref >= 0 ?     6.2831 +  kAref : kAref)	; distinction of positive and negative y and x
; calculation of distance- difference (m/s) and delaytime
kDdiff 	= sqrt((kx-krefx)^2 + (ky-krefy)^2 + (kz-krefz)^2  )			; calculation of the difference  of kreflection and ksound
kDist		= kDdiff + kDref
								; distance sourcesurface + distance surface listener
kDmtr 	= kDist * giunit					; calculation of difference in meters
kdel		= kDmtr / 0.345					; calculation of the delaytime [ms]
; reflection behind the source
kAdist	= abs(kA - kAref)
kEdist	= abs(kE - kEref)
kAdist	= (kAdist > 6.2831 ? kAdist-6.2831 : kAdist)		; reduces the angle kAdist to values between 0 and 6,2831
kmute		= kAdist + kEdist
kmute		= ( kmute < 1.5707 ? sin (kmute) : 1)		; if the reflection gets behind the source, it is muted
; calculation of attenuation (gain of delay)
kgain		= 1/kDist * idamp * kmute			; gain of early reflection according calculation of attenuation

; FILTERING UNIT
kDdiff	= ( kDdiff< 0.05	? 	0.05	: 	kDdiff)	; limit the distance source reflection
klfboost	= (sqrt(1 / kDdiff ) + kD) * ilfboost
afilt		tone		arefsig,  (sqrt(1/kDist)) * sr	; filtering of high frequencies according to distance
afilt		pareq		afilt, 174, klfboost, 0.7071, 1	; boosting low frequencies for souces near the wall
afilt		balance	afilt, arefsig				; balancing the output

; REFLECTING UNIT
girandpchmod = 0.8  				; amount of random pitch modulation	
              					; for the delay lines. 1 is the "normal"	
                				; amount, but this may be too high for		
                				; held pitches such as piano tones.		
           					; Adjust to taste.	
a$R.		randi	2, 3.973, $R.
adel		interp	kdel
adel 		= 		adel + a$R. * girandpchmod
asigref	vdelay3	afilt, adel, 200		; creating a reflection
asigref	=		asigref * kgain 	; assigning a gain

;ENCODE
;2nd order Ambisonics encoding equations	
kchnlw	= 	0.707107
kchnlx	=	cos(kAref) * cos(kEref)     
kchnly	=	sin(kAref) * cos(kEref)
kchnlz	=	sin(kEref)
kchnlr	=	1.5 * sin(kEref) * sin(kEref) -0.5
kchnls	=	cos(kAref) * sin(2*kEref)
kchnlt	=	sin(kAref) * sin(2*kEref)
kchnlu	=	cos(2*kAref) * cos(kEref) * cos(kEref)
kchnlv	=	sin(2*kAref) * cos(kEref) * cos(kEref)

; encoding of the global reverb signal
arw 		=   	asigref *	kchnlw	;(A,E)
arx		=	asigref *	kchnlx	;(A,E)
ary		=	asigref *	kchnly	;(A,E)
arz		= 	asigref *	kchnlz	;(A,E)
arr		=	asigref *	kchnlr	;(A,E)
ars		=	asigref *	kchnls	;(A,E)
art		=	asigref *	kchnlt	;(A,E)
aru		=	asigref *	kchnlu	;(A,E)
arv		=	asigref *	kchnlv	;(A,E)

asrw = asrw + arw				; store single reflection in global variables
asrx = asrx + arx
asry = asry + ary
asrz = asrz + arz
asrr = asrr + arr
asrs = asrs + ars
asrt = asrt + art
asru = asru + aru
asrv = asrv + arv

# 

$DIFFREFLECTION(1)
$DIFFREFLECTION(2)
$DIFFREFLECTION(3)
$DIFFREFLECTION(4)
$DIFFREFLECTION(5)
$DIFFREFLECTION(6)

#undef	DIFFREFLECTION(R)

;*********************************************************************************************************************************
;instr 75	; SPECULAR EARLY REFLECTIONS		conceived and coded 1999-2001 by Jan Jacob Hofmann,
;	using the Furse-Malham-Set (FMH) of encoding equations. Thanks to Dave Malham and Richard Furse. Thanks to Nora.
;	Also thanks to Peter Lennox for inspitation and pushing me in the right direction.
;				
;	Based on a Paper by David Griesinger published in the Proceedings of the 19th  AES International Conference, June 2001.
;*********************************************************************************************************************************
							; prolong duration to keep the reverb - tail
islfboost	= 1					; low frequency boost for sources near a wall
isdamp 		= 0.2					; damping factor of the reflecting surface
ishighdamp	= 15000					; absorbtion characteristics of the wall in respect to high frequencies: frequencies above ihighdamp get filtered out

;  Polar converter  -source as cartesian representation

kx 		= kD * cos(kA) * cos(kE)		; convert polar to carthesian representation
ky 		= kD * sin(kA) * cos(kE)		; convert polar to carthesian representation
kz 		= kD * sin(kE)				; convert polar to carthesian representation
kx		= (kx < 0.01 && kx > -0.01 ? 0.01 :kx)	; fix for that strange y-axis noise

iwall1	= 	gifrontwall			; assign distance of the wall
iwall2	= 	-gifrontwall			; assign distance of the wall
iwall3	= 	gisidewall			; assign distance of the wall
iwall4	= 	-gisidewall			; assign distance of the wall
iwall5	=	giceiling			; assign distance of the wall
iwall6	= 	-giceiling			; assign distance of the wall

ksdiff1  	= iwall1 - kx			; distance of reflection to the frontwall
ksdiff2  	= iwall2 - kx			; distance of reflection to the backwall
ksdiff3 	= iwall3 - ky			; distance of reflection to the left sidewall
ksdiff4 	= iwall4 - ky			; distance of reflection to the right sidewall
ksdiff5  	= iwall5 - kz			; distance of reflection to the ceiling
ksdiff6  	= iwall6 - kz			; distance of reflection to the floor

assig		tone	adelgrain, ishighdamp	; filtering of high frequencies due to absorbtion characteristics of the wall

#define	SPECREFLECTION(R)#

; calculation of points of reflection
iscnt		= $R.
kssum$R.	= abs(iwall$R.) + abs(ksdiff$R.)	; distance source  wall  listener (x,y,z-component)
ksrefx 	= 	(kx / kssum$R. ) * abs(iwall$R.)	; x- position of the reflection
ksrefy 	=   	(ky / kssum$R. ) * abs(iwall$R.)	; y- position of the reflection
ksrefz 	=    	(kz / kssum$R. ) * abs(iwall$R.)	; z- position of the reflection

; assignment of x-,y-,z- component at the wall 
ksrefx		= (iscnt = 1 ? iwall1	: 	ksrefx)	; asigning a position of the reflection at the x-axis
		if iscnt = 1	igoto	ycalc$R.
ksrefx		= (iscnt = 2 ? iwall2	: 	ksrefx)	; asigning a position of the reflection at the x-axis

ycalc$R.:
ksrefy		= (iscnt = 3 ? iwall3	: 	ksrefy)	; asigning a position of the reflection at the y-axis
		if iscnt = 3	igoto	zcalc$R.
ksrefy		= (iscnt = 4 ? iwall4	: 	ksrefy)	; asigning a position of the reflection at the y-axis

zcalc$R.:
ksrefz 		= (iscnt = 5 ? iwall5	: 	ksrefz)	; asigning a position of the reflection at the z-axis
		if iscnt = 5	igoto	labelout$R.
ksrefz		= (iscnt = 6 ? iwall6	: 	ksrefz)	; asigning a position of the reflection at the z-axis

labelout$R.:

ksrefx		= (ksrefx < 0.001 &&  ksrefx > -0.001	?	0.001  : ksrefx)	;  avoid zero to avoid division by zero  error (NaN) later
ksrefy		= (ksrefy < 0.001 &&  ksrefy > -0.001	?	0.001  : ksrefy)	;  avoid zero to avoid division by zero  error (NaN) later
ksrefz		= (ksrefz < 0.001 &&  ksrefz > -0.001	?	0.001  : ksrefz)	;  avoid zero to avoid division by zero  error (NaN) later

; A, E, D - calculation of the reflecting point

ksDref		= sqrt (ksrefx^2+ ksrefy^2+ ksrefz^2)	; calculates the distance of the reflection
ksxref		= ksrefx / ksDref			; this operation generates a vector equivalent at the unit sphere
ksyref		= ksrefy / ksDref			; this operation generates a vector equivalent at the unit sphere
kszref		= ksrefz / ksDref			; this operation generates a vector equivalent at the unit sphere

; Elevation calculation
ksEref		= cosinv ( sqrt( 1 - kszref^2))		; calculation of Elevation
ksEref		= (kszref< 0 ? -ksEref  : ksEref)	; if z is negative make Elevation- vector negative too

;  Angle calculation
ksAref		= sininv (ksyref/ cos (ksEref))				; calculation of Angle
ksAref		= (ksxref>=0 ? ksAref : 3.1415 - ksAref)		; distinction of positive and negative y
ksAref		= (ksyref<=0 &&	ksxref >= 0 ? 6.2831 + ksAref : ksAref)	; distinction of positive and negative y and x

; calculation of distance- difference (m/s) and delaytime
ksDdiff 	= sqrt((kx - ksrefx)^2 + (ky - ksrefy)^2 + (kz - ksrefz)^2)	; calculation of the difference  of kreflection and ksound
ksDist		= ksDdiff + ksDref				; distance sourcesurface + distance surface listener
ksDmtr 		= ksDist * giunit				; calculation of difference in meters
ksdel		= ksDmtr / 0.345				; calculation of the delaytime [ms]

; calculation of attenuation (gain of delay)
ksgain 	= 1/ksDist * isdamp				; gain of early reflection according calculation of attenuation

; reflection behind the source
ksAdist	= abs(kA - ksAref)
ksEdist	= abs(kE - ksEref)
ksAdist	= (ksAdist > 6.2831 ? ksAdist-6.2831 : ksAdist)		; reduces the angle kAdist to values between 0 and 6,2831
ksmute		= ksAdist + ksEdist
ksmute		= ( ksmute < 1.5707 ? sin (ksmute) : 1)		; if the reflection gets behind the source, it is muted
; filtering unit
ksDdiff	= ( ksDdiff< 0.05 ? 0.05 : ksDdiff)			; limit the distance source reflection
kslfboost = 	sqrt(1 / kDdiff) * ilfboost
asfilt		tone		assig, (sqrt(1 / ksDist)) * sr		; filtering of high frequencies according to distance
asfilt		pareq		asfilt, 174, kslfboost, 0.7071, 1	; boosting low frequencies for souces near the wall
asfilt		balance	asfilt, assig					; balancing the output

; reflecting unit
girandpchmod =0.8  			; amount of random pitch modulation	
                			; for the delay lines. 1 is the "normal"	
                			; amount, but this may be too high for		
                			; held pitches such as piano tones.		
                			; Adjust to taste.	
as$R.		randi	2, 3.973, $R.
asdel		interp	ksdel
asdel 		= 	asdel + as$R. * girandpchmod
assigref	vdelay3	asfilt, asdel, 200			; creation of a reflection 
assigref	=	assigref * ksgain * ksmute		; assigning a gain

;2nd order Ambisonics encoding equations	
kschnlw	= 	0.707107 
kschnlx	=	cos(ksAref) * cos(ksEref)     
kschnly	=	sin(ksAref) * cos(ksEref)
kschnlz	=	sin(ksEref)
kschnlr	=	1.5 * sin(ksEref) * sin(ksEref) -0.5
kschnls	=	cos(ksAref) * sin(2*ksEref)
kschnlt	=	sin(ksAref) * sin(2*ksEref)
kschnlu	=	cos(2*ksAref) * cos(ksEref) * cos(ksEref)
kschnlv	=	sin(2*ksAref) * cos(ksEref) * cos(ksEref)
       
; encoding of the audio signal
asw		= 	assigref *	kschnlw	;(A,E)
asx		=	assigref *	kschnlx	;(A,E)
asy		=	assigref *	kschnly	;(A,E)
asz		= 	assigref *	kschnlz	;(A,E)
asr		=	assigref *	kschnlr	;(A,E)
ass		=	assigref *	kschnls	;(A,E)
ast		=	assigref *	kschnlt	;(A,E)
asu		=	assigref *	kschnlu	;(A,E)
asv		=	assigref *	kschnlv	;(A,E)

assrw = assrw + asw				; store single reflection in global variables
assrx = assrx + asx
assry = assry + asy
assrz = assrz + asz
assrr = assrr + asr
assrs = assrs + ass
assrt = assrt + ast
assru = assru + asu
assrv = assrv + asv
	
#  

$SPECREFLECTION(1)
$SPECREFLECTION(2)
$SPECREFLECTION(3)
$SPECREFLECTION(4)
$SPECREFLECTION(5)
$SPECREFLECTION(6)

#undef	SPECREFLECTION(R)

			; generate B format
aw, ax, ay, az, ar, as, at, au, av bformenc adelgrain, kA, kE, 1, 1, 1      

aws = aw+assrw+asrw		;add dry signal to early reflections
axs = ax+assrx+asrx
ays = ay+assry+asry
azs = az+assrz+asrz
ars = ar+assrr+asrr
ass = as+assrs+asrs
ats = at+assrt+asrt
aus = au+assru+asru
avs = av+assrv+asrv


;	; decode B format for loudspeaker setup
;	**************************************************************************************
;quad Rig `Controlled Opposites' Decode Matrix (second Order)
;	**************************************************************************************
; This rig configuration produces an `in-phase' response and each speaker will have only a single maximum given a moving virtual sound source. 
;			w		x		y		z		r		s		t		u		v
; Speaker1<0.7071,0.7071,0.0000>  left front
;a1 	sum	aws *0.3536,	axs *0.2434,	ays * 0.2434, 										avs * 0.0964
; Speaker2<0.7071,-0.7071,0.0000>  right front
;a2	sum	aws *0.3536, 	axs *0.2434,	ays * -0.2434,										avs * -0.0964
; Speaker3<-0.7071,-0.7071,0.0000>  right back
;a3 	sum	aws *0.3536,	axs *-0.2434,	ays * -0.2434,										avs * 0.0964
; Speaker4 <-0.7071,0.7071,0.0000>  left back
;a4 	sum	aws *0.3536,	axs *-0.2434, 	ays * 0.2434,										avs * -0.0964
;outq a2, a3, a4,a1
	
;	**************************************************************************************
;cube: ;8chnl  decode (cube),2nd  order (second order controlled opposites)
;	**************************************************************************************
;  Rig `Controlled Opposites' Decode Matrix (Second Order)
;			w		x		y		z		r		s		t		u		v
; Speaker1	<0.5774,0.5774,-0.5774>  	front left lower
;a1 	sum	aws  *0.1768,	axs*0.114,	ays *0.114,	azs * -0.114,			ass*-0.0369,	ats * -0.0369,		avs * 0.0369	
; Speaker2	<0.5774,-0.5774,-0.5774>	front right lower
;a2 	sum	aws  *0.1768,	axs*0.114,	ays *-0.114,	azs * -0.114,			ass*-0.0369,	ats * 0.0369,		avs * -0.0369
; Speaker3	<-0.5774,-0.5774,-0.5774>	back right lower
;a3 	sum	aws  *0.1768,	axs*-0.114,	ays *-0.114,	azs * -0.114,			ass*0.0369,	ats * 0.0369,		avs * 0.0369
; Speaker4	<-0.5774,0.5774,-0.5774>	back left lower
;a4 	sum	aws  *0.1768,	axs*-0.114,	ays *0.114,	azs * -0.114,			ass*0.0369,	ats * -0.0369,		avs * -0.0369
; Speaker5	<0.5774,0.5774,0.5774>		front left upper
;a5 	sum	aws  *0.1768,	axs*0.114,	ays *0.114,	azs * 0.114,			ass*0.0369,	ats * 0.0369,		avs * 0.0369
; Speaker6	<0.5774,-0.5774,0.5774>		front right upper
;a6 	sum	aws  *0.1768,	axs*0.114,	ays *-0.114,	azs * 0.114,			ass*0.0369,	ats * -0.0369,		avs * -0.0369
; Speaker7	<-0.5774,-0.5774,0.5774>	back  right upper
;a7 	sum	aws  *0.1768,	axs*-0.114,	ays *-0.114,	azs * 0.114,			ass*-0.0369,	ats * -0.0369,		avs * 0.0369
; Speaker8	<-0.5774,0.5774,0.5774>		back left upper
;a8 	sum	aws  *0.1768,	axs*-0.114,	ays *0.114,	azs * 0.114,			ass*-0.0369,	ats * 0.0369,		avs * -0.0369
;outo a1, a2, a3, a4, a5, a6, a7, a8 

;	**************************************************************************************
;octogan2 8chnl  decode ,2nd  order (second order controlled opposites)
;a1 = front centre speaker, count counterclockwise, 
;	**************************************************************************************
; Rig `Controlled Opposites' Decode Matrix (Second Order)
; This rig configuration produces an `in-phase' response and each speaker will have only a single maximum given a moving virtual sound source.
;			w		x		y		z		r		s		t		u		v
; Speaker 1	<1.0000,0.0000,0.0000> 
;  a1 	sum	aws *0.1768,	axs * 0.1721,										aus * 0.0482
; Speaker 2	<0.7071,0.7071,0.0000>
;  a2 	sum	aws *0.1768,	axs *0.1217,	ays * 0.1217,										avs * 0.0482
; Speaker 3	<0.0000,1.0000,0.0000>
;  a3 	sum	aws * 0.1768,			ays * 0.1721,								aus * -0.0482
; Speaker 4	<-0.7071,0.7071,0.0000>
;  a4 	sum	aws *0.1768,	axs *-0.1217, 	ays * 0.1217,										avs * -0.0482
; Speaker 5	<-1.0000,0.0000,0.0000>
;  a5 	sum	aws *0.1768,	axs * -0.1721, 										aus * 0.0482
; Speaker 6	<-0.7071,-0.7071,0.0000>
;  a6 	sum	aws *0.1768,	axs *-0.1217, 	ays * -0.1217,										avs * 0.0482
; Speaker 7	<0.0000,-1.0000,0.0000>
;  a7 	sum	aws * 0.1768,			ays * -0.1721,								aus * -0.0482
; Speaker 8	<0.7071,-0.7071,0.0000>
;  a8 	sum	aws *0.1768,	axs *0.1217,	ays * -0.1217,										avs * -0.0482
;outo a1, a2, a3, a4, a5, a6, a7, a8 

;	**************************************************************************************
; octagon1 8chnl  decode 2nd  order (second order controlled opposites)
;a1 = front left speaker, count counterclockwise, no speaker centre front.
;	**************************************************************************************
;			w		x		y		z		r		s		t		u		v
; Speaker 1	<0.9239,0.3827,0.0000>
 a1 	sum	aws *0.1768, axs *0.1591,	ays * 0.0659,								aus *0.0342,	avs * 0.0342
; Speaker 2	<0.3827,0.9239,0.0000>
 a2 	sum	aws *0.1768,	axs *0.0659,	ays * 0.1591,								aus *-0.0342, avs * 0.0342
; Speaker 3	<-0.3827,0.9239,0.0000>
 a3	sum	aws *0.1768,	axs *-0.0659, ays * 0.1591,								aus *-0.0342,avs * -0.0342
; Speaker 4	<-0.9239,0.3827,0.0000>
 a4 	sum	aws *0.1768,	axs *-0.1591, ays * 0.0659,								aus *0.0342,	avs * -0.0342
; Speaker 5 	<-0.9239,-0.3827,0.0000>
 a5	sum	aws *0.1768,	axs *-0.1591, ays * -0.0659,								aus *0.0342,	avs * 0.0342
; Speaker 6 	<-0.3827,-0.9239,0.0000>
 a6	sum	aws *0.1768,	axs *-0.0659, ays * -0.1591,								aus *-0.0342,avs * 0.0342
; Speaker 7 	<0.3827,-0.9239,0.0000>
 a7	sum	aws *0.1768,	axs *0.0659,	ays * -0.1591,								aus *-0.0342, avs * -0.0342
; Speaker 8 	<0.9239,-0.3827,0.0000>
 a8 	sum	aws *0.1768,	axs *0.1591,	ays * -0.0659,								aus *0.0342,	avs * -0.0342
outo a1, a2, a3, a4, a5, a6, a7, a8 



gamix= agrainr+gamix	; global reverb send
assrw = 0				
assrx = 0
assry = 0
assrz = 0
assrr = 0
assrs = 0
assrt = 0
assru = 0
assrv = 0

asrw = 0				
asrx = 0
asry = 0
asrz = 0
asrr = 0
asrs = 0
asrt = 0
asru = 0
asrv = 0

endin


instr 99
;*****************************************************************************************
; SPATIAL GLOBAL REVERB  conceived and coded 1999-2005 by Jan Jacob Hofmann
; using the Furse-Malham-Set (FMH) of encoding equations. 
;
; 8 delay line FDN reverb, with feedback matrix based upon 
; physical modeling scattering junction of 8 lossless waveguides
; of equal characteristic impedance. Based on Julius O. Smith III, 
; "A New Approach to Digital Reverberation using Closed Waveguide
; Networks," Proceedings of the International Computer Music 
; Conference 1985, p. 47-53 (also available as a seperate
; publication from CCRMA), as well as some more recent papers by
; Smith and others.
;
; Coded by Sean Costello, October 1999/ modified by Jan Jacob Hofmann August 2005
;*****************************************************************************************
arevsig	= gamix		; obtain reverb send signal from grain instrument
			; arevsig is the global input to the reverb.	
awr init 0
axr init 0
ayr init 0
azr init 0
arr init 0
asr init 0
atr init 0
aur init 0
avr init 0	
		
afilt1	init	0
afilt2	init	0
afilt3	init	0
afilt4	init	0
afilt5	init	0
afilt6	init	0
afilt7	init	0
afilt8	init	0
afilt9	init	0
afilt10	init	0
afilt11	init	0
afilt12	init	0
		
; Delay times chosen to be prime numbers. Works with sr=44100/ sr=48000 ONLY. If you wish to		
; use a different delay time, find some new prime numbers that will give roughly the		
; same delay times for the new sampling rate, or adjust to taste.
		
if sr = 48000		goto	delaytime48000		; delaytime chosen for sr=44100	
idel1 =  	(1237.000 / sr)		
idel2 = 	(1381.000 / sr) 			
idel3 =  	(1609.000 / sr)			
idel4 =  	(1777.000 / sr)		
idel5 =  	(1951.000 / sr)				
idel6 =  	(2063.000 / sr)			
idel7 =  	(1069.000 / sr)	 		
idel8 =  	( 967.000 / sr)					
idel9 =  	(1889.000 / sr)				
idel10 =  	(1693.000 / sr)					
idel11 =  	(1453.000 / sr)					
idel12 =  	(1291.000 / sr)
			goto		contin
delaytime48000:							; delaytime for sr=48000		
idel1 =  	(1361.000/sr)		
idel2 = 	(1499.000/sr) 			
idel3 =  	(1753.000/sr)			
idel4 =  	(1933.000/sr)		
idel5 =  	(2129.000/sr)				
idel6 =  	(2243.000/sr)			
idel7 =  	(1163.000/sr)	 		
idel8 =  	(1051.000/sr)					
idel9 =  	(2053.000/sr)				
idel10 =  	(1693.000/sr)					
idel11 =  	(1847.000/sr)					
idel12 =  	(1409.000/sr)

contin:				
	
igain 		= girvbdecay	; gain of reverb. Adjust empirically	
               			; for desired reverb time. .6 gives	
               			; a good small "live" room sound, .8	
               			; a small hall, .9 a large hall,	
               			; .99 an enormous stone cavern.	
	
ipitchmod 	= girandpchmod 	; amount of random pitch modulation	
                		; for the delay lines. 1 is the "normal"	
                		; amount, but this may be too high for		
                		; held pitches such as piano tones.		
                		; Adjust to taste.		
		
itone 		= gicutofffreq  ; Cutoff frequency of lowpass filters		
                		; in feedback loops of delay lines,		
                		; in Hz. Lower cutoff frequencies results		
                		; in a sound with more high-frequency		
                		; damping.		
		
; k1-k8 are used to add random pitch modulation to the		
; delay lines. Helps eliminate metallic overtones		
; in the reverb sound.		
		
k1	randi	.001, 3.1, .06
k2	randi	.0011, 3.5, .9
k3	randi	.0017, 1.11, .7
k4	randi	.0006, 3.973, .3
k5	randi	.001, 2.341, .63
k6	randi	.0011, 1.897, .17
k7	randi	.0017, 0.891, .19
k8	randi	.0006, 3.221, .44
k9	randi	.001, 1.891, .88
k10	randi	.0011, 2.317, .95
k11	randi	.0017, 1.091, .36
k12	randi	.0006, 2.821, .41
		
; apj is used to calculate "resultant junction pressure" for		
; the scattering junction of e.g. 8 lossless waveguides		
; of equal characteristic impedance. If you wish to		
; add more delay lines, simply add them to the following		
; equation, and replace the .25 by 2/N, where N is the		
; number of delay lines.		
		
apj = .16666666667 * (afilt1 + afilt2 + afilt3 + afilt4 + afilt5 + afilt6 +afilt7 + afilt8 + afilt9 + afilt10 +afilt11 + afilt12)		
				
adum1		delayr	1
adel1		deltapi	idel1 + k1 * ipitchmod
		delayw	arevsig  + apj - afilt1
		
adum2		delayr	1
adel2		deltapi	idel2 + k2 * ipitchmod
		delayw	arevsig   + apj - afilt2
		
adum3		delayr	1
adel3		deltapi	idel3 + k3 * ipitchmod
		delayw	arevsig   + apj - afilt3
		
adum4		delayr	1
adel4		deltapi	idel4 + k4 * ipitchmod
		delayw	arevsig  + apj - afilt4
		
adum5		delayr	1
adel5		deltapi	idel5 + k5 * ipitchmod
		delayw	arevsig   + apj - afilt5
		
adum6		delayr	1
adel6		deltapi	idel6 + k6 * ipitchmod
		delayw	arevsig   + apj - afilt6
		
adum7		delayr	1
adel7		deltapi	idel7 + k7 * ipitchmod
		delayw	arevsig   + apj - afilt7
		
adum8		delayr	1
adel8		deltapi	idel8 + k8 * ipitchmod
		delayw	arevsig   + apj - afilt8
			
adum9		delayr	1
adel9		deltapi	idel9 + k9 * ipitchmod
		delayw	arevsig   + apj - afilt9
			
adum10	delayr	1
adel10	deltapi	idel10 + k10 * ipitchmod
		delayw	arevsig   + apj - afilt10
			
adum11	delayr	1
adel11	deltapi	idel11 + k11 * ipitchmod
		delayw	arevsig   + apj - afilt11
			
adum12	delayr	1
adel12	deltapi	idel12 + k12 * ipitchmod
		delayw	arevsig   + apj - afilt12
		
; 1st order lowpass filters in feedback	loops of delay lines.		
		
afilt1	tone	adel1  * igain, itone
afilt2	tone	adel2  * igain, itone
afilt3	tone	adel3  * igain, itone
afilt4	tone	adel4  * igain, itone
afilt5	tone	adel5  * igain, itone
afilt6	tone	adel6  * igain, itone
afilt7	tone	adel7  * igain, itone
afilt8	tone	adel8  * igain, itone
afilt9	tone	adel9  * igain, itone
afilt10	tone	adel10 * igain, itone
afilt11	tone	adel11 * igain, itone
afilt12	tone	adel12 * igain, itone

; kA1-kA12 are the angles pointing to the 12 corners of an ikosaeder
; kE1-kE12 is the elevation pointing to the 12 corners of an ikosaeder

iA1 	=	0			;  angle reverberant point 1 
iA2 	= 	0.6238			;  angle reverberant point 2
iA3 	= 	1.2566			;  angle reverberant point 3
iA4 	=	1.8849			;  angle reverberant point 4
iA5 	=	2.5132			;  angle reverberant point 5
iA6 	=	3.1415			;  angle reverberant point 6
iA7 	=	3.7699			;  angle reverberant point 7
iA8 	=	4.3982			;  angle reverberant point 8
iA9 	=	5.0265			;  angle reverberant point 9
iA10 	= 	5.6548			;  angle reverberant point 10
iA11 	= 	0			;  angle reverberant point 11
iA12 	= 	0			;  angle reverberant point 12


iE1 	=	 0.463646		;  height reverberant point 1  ( 26.565)
iE2 	=	-0.463646		;  height reverberant point 2  ( -26.565)
iE3 	=	0.463646		;  height reverberant point 3  ( 26.565)
iE4 	=	-0.463646		;  height reverberant point 4  ( -26.565)
iE5 	=	0.463646		;  height reverberant point 5  ( 26.565)
iE6 	=	-0.463646		;  height reverberant point 6  ( -26.565)
iE7 	= 	0.463646		;  height reverberant point 7  ( 26.565)
iE8 	=	-0.463646		;  height reverberant point 8  ( -26.565)
iE9 	=	0.463646		;  height reverberant point 9  ( 26.565)
iE10	=	-0.463646		;  height reverberant point 10 ( -26.565)
iE11	= 	1.570796		;  height reverberant point 11 ( 90)
iE12	=	-1.570796		;  height reverberant point 12 ( -90)

#define	GLOBALREVERBENCODING(Z) #

kchnlw	= 	0.707107
kchnlx	=	cos(iA$Z.) * cos(iE$Z.)     
kchnly	=	sin(iA$Z.) * cos(iE$Z.)
kchnlz	=	sin(iE$Z.)
kchnlr	=	1.5 * sin(iE$Z.) * sin(iE$Z.) -0.5
kchnls	=	cos(iA$Z.) * sin(2 * iE$Z.)
kchnlt	=	sin(iA$Z.) * sin(2 * iE$Z.)
kchnlu	=	cos(2 * iA$Z.) * cos(iE$Z.) * cos(iE$Z.)
kchnlv	=	sin(2 * iA$Z.) * cos(iE$Z.) * cos(iE$Z.)
   
awr		= awr + afilt$Z. *	kchnlw	;(A,E)
axr		= axr +	afilt$Z. *	kchnlx	;(A,E)
ayr		= ayr +	afilt$Z. *	kchnly	;(A,E)
azr		= azr +	afilt$Z. *	kchnlz	;(A,E)
arr		= arr +	afilt$Z. *	kchnlr	;(A,E)
asr 		= asr +	afilt$Z. *	kchnls	;(A,E)
atr		= atr +	afilt$Z. *	kchnlt	;(A,E)
aur		= aur +	afilt$Z. *	kchnlu	;(A,E)
avr		= avr +	afilt$Z. *	kchnlv	;(A,E)

#

$GLOBALREVERBENCODING(1)		
$GLOBALREVERBENCODING(2)		
$GLOBALREVERBENCODING(3)		
$GLOBALREVERBENCODING(4)		
$GLOBALREVERBENCODING(5)		
$GLOBALREVERBENCODING(6)		
$GLOBALREVERBENCODING(7)
$GLOBALREVERBENCODING(8)
$GLOBALREVERBENCODING(9)
$GLOBALREVERBENCODING(10)
$GLOBALREVERBENCODING(11)
$GLOBALREVERBENCODING(12)

#undef	GLOBALREVERBENCODING(Z)


;	; decode B format for loudspeaker setup
;	**************************************************************************************
;quad Rig `Controlled Opposites' Decode Matrix (second Order)
;	**************************************************************************************
; This rig configuration produces an `in-phase' response and each speaker will have only a single maximum given a moving virtual sound source. 
;			w		x		y		z		r		s		t		u		v
; Speaker1<0.7071,0.7071,0.0000>  left front
;a1 	sum	awr *0.3536,	axr *0.2434,	ayr * 0.2434, 										avr * 0.0964
; Speaker2<0.7071,-0.7071,0.0000>  right front
;a2	sum	awr *0.3536, 	axr *0.2434,	ayr * -0.2434,										avr * -0.0964
; Speaker3<-0.7071,-0.7071,0.0000>  right back
;a3 	sum	awr *0.3536,	axr *-0.2434,	ayr * -0.2434,										avr * 0.0964
; Speaker4 <-0.7071,0.7071,0.0000>  left back
;a4 	sum	awr *0.3536,	axr *-0.2434, 	ayr * 0.2434,										avr * -0.0964
;outq a2,a1,a3,a4
	
;	**************************************************************************************
;cube: ;8chnl  decode (cube),2nd  order (second order controlled opposites)
;	**************************************************************************************
;  Rig `Controlled Opposites' Decode Matrix (Second Order)
;			w		x		y		z		r		s		t		u		v
; Speaker1	<0.5774,0.5774,-0.5774>  	front left lower
;a1 	sum	awr  *0.1768,	axr*0.114,	ayr *0.114,	azr * -0.114,			asr*-0.0369,	atr * -0.0369,		avr * 0.0369	
; Speaker2	<0.5774,-0.5774,-0.5774>	front right lower
;a2 	sum	awr  *0.1768,	axr*0.114,	ayr *-0.114,	azr * -0.114,			asr*-0.0369,	atr * 0.0369,		avr * -0.0369
; Speaker3	<-0.5774,-0.5774,-0.5774>	back right lower
;a3 	sum	awr  *0.1768,	axr*-0.114,	ayr *-0.114,	azr * -0.114,			asr*0.0369,	atr * 0.0369,		avr * 0.0369
; Speaker4	<-0.5774,0.5774,-0.5774>	back left lower
;a4 	sum	awr  *0.1768,	axr*-0.114,	ayr *0.114,	azr * -0.114,			asr*0.0369,	atr * -0.0369,		avr * -0.0369
; Speaker5	<0.5774,0.5774,0.5774>		front left upper
;a5 	sum	awr  *0.1768,	axr*0.114,	ayr *0.114,	azr * 0.114,			asr*0.0369,	atr * 0.0369,		avr * 0.0369
; Speaker6	<0.5774,-0.5774,0.5774>		front right upper
;a6 	sum	awr  *0.1768,	axr*0.114,	ayr *-0.114,	azr * 0.114,			asr*0.0369,	atr * -0.0369,		avr * -0.0369
; Speaker7	<-0.5774,-0.5774,0.5774>	back  right upper
;a7 	sum	awr  *0.1768,	axr*-0.114,	ayr *-0.114,	azr * 0.114,			asr*-0.0369,	atr * -0.0369,		avr * 0.0369
; Speaker8	<-0.5774,0.5774,0.5774>		back left upper
;a8 	sum	awr  *0.1768,	axr*-0.114,	ayr *0.114,	azr * 0.114,			asr*-0.0369,	atr * 0.0369,		avr * -0.0369
;outo a1, a2, a3, a4, a5, a6, a7, a8 


;	**************************************************************************************
;octo:	; 8chnl  decode (octagon 2),2nd  order (second order controlled opposites)
;a1 = front centre speaker, count counterclockwise, 
;	**************************************************************************************
; Rig `Controlled Opposites' Decode Matrix (Second Order)
; This rig configuration produces an `in-phase' response and each speaker will have only a single maximum given a moving virtual sound source.
;			w		x		y		z		r		s		t		u		v
; Speaker 1	<1.0000,0.0000,0.0000> 
;  a1 	sum	awr *0.1768,	axr * 0.1721,										aur * 0.0482
; Speaker 2	<0.7071,0.7071,0.0000>
;  a2 	sum	awr *0.1768,	axr *0.1217,	ayr * 0.1217,										avr * 0.0482
; Speaker 3	<0.0000,1.0000,0.0000>
;  a3 	sum	awr * 0.1768,			ayr * 0.1721,								aur * -0.0482
; Speaker 4	<-0.7071,0.7071,0.0000>
;  a4 	sum	awr *0.1768,	axr *-0.1217, 	ayr * 0.1217,										avr * -0.0482
; Speaker 5	<-1.0000,0.0000,0.0000>
;  a5 	sum	awr *0.1768,	axr * -0.1721, 										aur * 0.0482
; Speaker 6	<-0.7071,-0.7071,0.0000>
;  a6 	sum	awr *0.1768,	axr *-0.1217, 	ayr * -0.1217,										avr * 0.0482
; Speaker 7	<0.0000,-1.0000,0.0000>
;  a7 	sum	awr * 0.1768,			ayr * -0.1721,								aur * -0.0482
; Speaker 8	<0.7071,-0.7071,0.0000>
;  a8 	sum	awr *0.1768,	axr *0.1217,	ayr * -0.1217,										avr * -0.0482
;outo a1, a2, a3, a4, a5, a6, a7, a8 

;	**************************************************************************************
; octagon1 8chnl  decode 2nd  order (second order controlled opposites)
;a1 = front left speaker, count counterclockwise, no speaker centre front.
;	**************************************************************************************
;			w		x		y		z		r		s		t		u		v
; Speaker 1	<0.9239,0.3827,0.0000>
 a1 	sum	awr *0.1768, axr *0.1591,	ayr * 0.0659,								aur *0.0342,	avr * 0.0342
; Speaker 2	<0.3827,0.9239,0.0000>
 a2 	sum	awr *0.1768,	axr *0.0659,	ayr * 0.1591,								aur *-0.0342, avr * 0.0342
; Speaker 3	<-0.3827,0.9239,0.0000>
 a3	sum	awr *0.1768,	axr *-0.0659, ayr * 0.1591,								aur *-0.0342,avr * -0.0342
; Speaker 4	<-0.9239,0.3827,0.0000>
 a4 	sum	awr *0.1768,	axr *-0.1591, ayr * 0.0659,								aur *0.0342,	avr * -0.0342
; Speaker 5 	<-0.9239,-0.3827,0.0000>
 a5	sum	awr *0.1768,	axr *-0.1591, ayr * -0.0659,								aur *0.0342,	avr * 0.0342
; Speaker 6 	<-0.3827,-0.9239,0.0000>
 a6	sum	awr *0.1768,	axr *-0.0659, ayr * -0.1591,								aur *-0.0342,avr * 0.0342
; Speaker 7 	<0.3827,-0.9239,0.0000>
 a7	sum	awr *0.1768,	axr *0.0659,	ayr * -0.1591,								aur *-0.0342, avr * -0.0342
; Speaker 8 	<0.9239,-0.3827,0.0000>
 a8 	sum	awr *0.1768,	axr *0.1591,	ayr * -0.0659,								aur *0.0342,	avr * -0.0342
outo a1, a2, a3, a4, a5, a6, a7, a8 




gamix=0
awr = 0
axr = 0
ayr = 0
azr = 0
arr = 0
asr = 0
atr = 0
aur = 0
avr = 0
endin

